package writer2latex.xhtml;

import java.util.LinkedList;



import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import writer2latex.office.XMLString;
import static writer2latex.util.Misc.*;


//LinkedList<String>   stringList = new LinkedList<String>();
public class GreenstoneTags {

	private static final String NONE = "none";
	private static final String SECTIONS = "sections";
	private static final String DIV = "div";
	
	
	private static LinkedList<Integer> headerStack = new LinkedList<Integer>();
	private static boolean pageOpened = false; 
	//headings none
	private static String headingSeparation = "sections";
	//sections div none
	private static String pageSeparation = "sections";

	protected static Node processHeading(Node currentNode, Node hnode, int pageNum) {
		
		//Get outline level
		String sLevel = getAttribute(currentNode, XMLString.TEXT_OUTLINE_LEVEL);
		String title = getTitle(currentNode).trim();
		if (sLevel == null || sLevel.isEmpty()) {
			return hnode;
		}
		if (title == null || title.isEmpty()) {
			return hnode;
		}
		int nLevel = Integer.parseInt(sLevel);
		
		if (pageOpened) {
			hnode = closePage(hnode);
		}
		
		if (headingSeparation.equals(SECTIONS)){
			closeHeadingSections(hnode, nLevel);
			
			openHeadingComment(hnode, title);
			headerStack.offerFirst(Integer.parseInt(sLevel));
		}
		if (!noPageSeparation() && !noHeadingSeparation()){
			hnode = openPage(hnode, pageNum);
		}
		
		return hnode;
	}
	
	protected static Node processPageBreak(Node currentNode, Node hnode, Integer pageNum){
		if (noPageSeparation()){
			return hnode;
		}
		if (pageOpened) {
			hnode = closePage(hnode);
		}
		hnode = openPage(hnode, pageNum);

		return hnode;
	}
	
	/**
	 * Opens main document section heading tag
	 */
	protected static Node StartDocument(Node hnode, String title, String heading, String pages, int pageNum){
		headingSeparation = heading;
		pageSeparation = pages;
		
		if (noHeadingSeparation() && noPageSeparation()){
			return hnode;
		}
		if(headingSeparation.equals(SECTIONS)){
			//Create global section
			openHeadingComment(hnode, title);
		}
		hnode = openPage(hnode, pageNum);
		
		return hnode;
	}
	//Method to close open tags at the end of the document
	protected static Node endDocument(Node hnode){
		if (noHeadingSeparation() && noPageSeparation()){
			return hnode;
		}
		if (pageOpened){
			hnode = closePage(hnode);
		}
		if (headingSeparation.equals(SECTIONS)){
			closeHeadingSections(hnode, 0);
			//Close global section
			addCloseComment(hnode);
		} 
		
		return hnode;
	}
	
	private static Node openPageDiv(Node node,int pageNum){
		if (node == null){
			return node;
		}
		Document doc = node.getOwnerDocument();
		Element openBlock = (Element) doc.createElement("div");
		openBlock.setAttribute("class", "pageNum");
		openBlock.setAttribute("page", Integer.toString(pageNum));
		// insert open section comment before header node
		node.appendChild((Node)openBlock);
		node = openBlock;
		return openBlock;
	}
	
	private static Node exitPageDiv(Node node){
		
		while (	!isRoot(node)	&& !isElement("div", node) ){
			node = node.getParentNode();
		}
		Node result = node.getParentNode();
		if (node.getChildNodes().getLength() == 0){
			result.removeChild(node);
		}
		
		return result;
	}
	
	private static String getTitle(Node currentNode) {
		Node content = currentNode.cloneNode(true);
		NodeList contentNodes = content.getChildNodes();
		String title = null;

		int i = 0;
		while (i < contentNodes.getLength()) {
			Node child = contentNodes.item(i);
			if (isElement(child) ){
				if (child.getNodeName().equals(XMLString.TEXT_TAB) ||
					child.getNodeName().equals(XMLString.TEXT_LINE_BREAK)	){
					Document doc = child.getOwnerDocument();
					Node testSpace = doc.createTextNode(" ");
					content.insertBefore(testSpace, child);
					content.removeChild(child);
				}
			}
				i++;
		} 
		NodeList notes = ((Element) content).getElementsByTagName(XMLString.TEXT_NOTE);
		int j = 0;
		while (j < notes.getLength()){
			Node note = notes.item(j);
			note.getParentNode().removeChild(note);
		}
		title = content.getTextContent();
		return title;
	}

	
	private static void openPageComment(Node hnode, Integer pageNum){
		Document doc = hnode.getOwnerDocument();
		Node openSection = doc.createComment(openPageCommentText(pageNum));
		// insert open section comment before header node
		hnode.appendChild(openSection);
	}
	
	private static void openHeadingComment(Node hnode, String title){
		Document doc = hnode.getOwnerDocument();
		Node openSection = doc.createComment(openHeadingCommentText(title));
		// insert open section comment before header node
		hnode.appendChild(openSection);
	}

	private static void addCloseComment(Node node){
		Document doc = node.getOwnerDocument();
		Node closeSection = doc.createComment("</Section>");
		//insert open section comment before header node
		node.appendChild(closeSection);
	}
	
	private static void closeHeadingSections(Node hnode, int nLevel){
		if (headerStack.isEmpty()) {
			return;
		}
		//Close all sections with level less than current
		while (nLevel <= headerStack.peek()) {
			addCloseComment(hnode);
			headerStack.poll();
			if (headerStack.isEmpty()) {
				break;
			}
		}
	}
	protected static Node closePage(Node hnode){
		if (pageSeparation.equals(SECTIONS)){
			//If section is empty. In case we are closing section
			// the last comment is opened page section
			if (hnode.getLastChild().getNodeType() == Node.COMMENT_NODE){
				hnode.removeChild(hnode.getLastChild());
			} else {
				addCloseComment(hnode);	
			}
		} else if (pageSeparation.equals(DIV)){
			hnode = exitPageDiv(hnode);
		}
		pageOpened = false;
		return hnode;
	}
	protected static Node openPage(Node hnode, Integer pageNum){
		if (pageSeparation.equals(SECTIONS)){
			openPageComment(hnode, pageNum);
			pageOpened = true;
		} 
		else if (pageSeparation.equals(DIV)){
			hnode = openPageDiv(hnode, pageNum);
			pageOpened = true;
		} 
		
		return hnode;
	}
	private static boolean noHeadingSeparation() {
		return headingSeparation.equals(NONE);
	}
	private static boolean noPageSeparation() {
		return pageSeparation.equals(NONE);
	}
	private static String openHeadingCommentText(String title) {
		String comment = "<Section>\n<Description>\n<Metadata name=\"Title\">" + title
				+ "</Metadata>\n</Description>";
		return comment;
	}
	private static String openPageCommentText(Integer pageNum) {
		String comment = "<Section>\n<Description>\n<Metadata name=\"Title\">" + pageNum
				+ "</Metadata>\n<Metadata name=\"Page\">" + pageNum + "</Metadata>\n</Description>";
		return comment;
	}

}
	